import os
import pathlib
import subprocess as sp

import ruamel.yaml
from box import Box

import dandori.env
import dandori.exception
import dandori.log
import dandori.process

L = dandori.log.get_logger(__name__)


class Operation:
    def fail(self, message):
        """Fail action with some message"""
        raise dandori.exception.Failure(message)

    def cancel(self, message):
        """Cancel action with some message"""
        raise dandori.exception.Cancel(message)

    def run(self, args, **kwargs):
        """subprocess wrapper"""
        if "encoding" not in kwargs:
            kwargs["encoding"] = "utf-8"
        kwargs.setdefault("check", True)
        # Always inherit env vars
        env = os.environ.copy()
        env.update(kwargs.get("env", {}))
        kwargs["env"] = env
        try:
            L.verbose3("Execute: %s", args)
            return dandori.process.run(args, **kwargs)
        except sp.CalledProcessError as e:
            L.error("Finished with code=%d: %s", e.returncode, args)
            raise

    def run_venv(self, *args, python_path="python", name="venv", **kwargs):
        """Run command with virtualenv"""
        env = self._prepare_venv(python_path, name)
        kwargs.setdefault("env", {}).update(env)
        return self.run(*args, **kwargs)

    def parse_toml(self, path: str, encoding="utf-8"):
        """Parse toml file"""
        return Box.from_toml(filename=path, encoding=encoding)

    def parse_yaml(self, path: str, encoding="utf-8"):
        """Parse toml file"""
        return Box.from_yaml(filename=path, encoding=encoding)

    def dump_toml(self, obj, path: str, encoding="utf-8"):
        """Dump toml file"""
        Box(obj).to_toml(filename=path, encoding=encoding)

    def dump_yaml(self, obj, path: str, encoding="utf-8"):
        """Dump toml file"""
        if isinstance(obj, (dict, Box)):
            Box(obj).to_yaml(filename=path, encoding=encoding)
        else:
            yaml = ruamel.yaml.YAML()
            with open(path, "w", encoding=encoding) as fo:
                yaml.dump(obj, fo)

    def _prepare_venv(self, python_path, name):
        venv_dir = pathlib.Path(dandori.env.tempdir().name) / name
        if not venv_dir.exists():
            self.run([python_path, "-m", "venv", "--clear", "--symlinks", str(venv_dir)])
        if not venv_dir.exists():
            raise dandori.exception.Failure(f"Virtualenv directory does not exist: {venv_dir}")
        return {
            "VIRTUAL_ENV": str(venv_dir),
            "PATH": f"{venv_dir}/bin:{os.environ['PATH']}",
        }



"""Modified version of https://github.com/pycontribs/subprocess-tee

Copyright (c) 2020 Sorin Sbarnea
Released under the MIT license
http://opensource.org/licenses/mit-license.php

"""
from __future__ import annotations

import asyncio
import subprocess as sp
import typing as T

STREAM_LIMIT = 2 ** 23  # 8MB instead of default 64kb, override it if you need


async def _read_stream(stream, outlist, echo, encoding):
    while True:
        line = await stream.readline()
        if line:
            if encoding is not None:
                line = line.decode(encoding)
            outlist.append(line)
            if echo:
                print(line, end="")
        else:
            break


async def _feed_stdin(stdin, ipt):
    stdin.write(ipt)
    try:
        await stdin.drain()
    finally:
        stdin.close()


async def _stream_subprocess(args, echo=True, **kwargs) -> sp.CompletedProcess:
    kwargs.pop("stdout", None)
    kwargs.pop("stderr", None)
    kwargs.setdefault("limit", STREAM_LIMIT)
    kwargs["stdout"] = asyncio.subprocess.PIPE
    kwargs["stderr"] = asyncio.subprocess.STDOUT
    encoding = kwargs.pop("encoding", "utf-8")
    input_str = kwargs.pop("input", None)
    if input_str:
        kwargs["stdin"] = asyncio.subprocess.PIPE
    if kwargs.get("shell", False):
        proc = await asyncio.create_subprocess_shell(args, **kwargs)
    else:
        proc = await asyncio.create_subprocess_exec(*args, **kwargs)

    try:
        loop = asyncio.get_event_loop()
        tasks = []
        if input_str:
            if isinstance(input_str, str):
                input_str = input_str.encode("utf-8")
            tasks.append(loop.create_task(_feed_stdin(proc.stdin, input_str)))
        out: list[str] = []
        tasks.append(loop.create_task(_read_stream(proc.stdout, out, echo, encoding)))
        await asyncio.wait(tasks)

        output = ""
        if out:
            output = "".join(out)

        return sp.CompletedProcess(
            args=args,
            returncode=await proc.wait(),
            stdout=output,
            stderr="",
        )
    except BaseException:
        proc.kill()
        raise


def run(args: T.Union[str, list[str]], echo=True, **kwargs) -> sp.CompletedProcess:
    """Always capture

    echo(default: True) <- print to stdout or not
    """
    check = kwargs.pop("check", False)

    loop = asyncio.get_event_loop()
    result = loop.run_until_complete(_stream_subprocess(args, echo=echo, **kwargs))
    if check and result.returncode != 0:
        raise sp.CalledProcessError(result.returncode, args, output=result.stdout, stderr=result.stderr)
    return result



from __future__ import annotations

import contextlib
import importlib.machinery
import sys
import typing as T

from box import Box

import dandori.response

from . import env, exception, log
from .config import ConfigLoader
from .context import Context
from .gh import GitHub, GitHubMock
from .ops import Operation

L = log.get_logger(__name__)


class HandlerFinder(importlib.machinery.PathFinder):
    @classmethod
    def find_spec(cls, fullname, path=None, target=None):
        """dynamic loading of dandori.handlers"""
        L.debug("Dynamic module finder: %s, %s, %s", fullname, path, target)
        if fullname == "dandori.handlers":
            path = [env.tempdir().name]
            spec = importlib.machinery.PathFinder.find_spec(fullname, path, target)
            return spec
        return None


class Runner:
    """Running some with user configuration"""

    def __init__(self, path, options: Box, local_mode=False):
        """Running some user defined function"""
        self._cfg_path = path
        self._options = options
        self._local_mode = local_mode

    def execute(self, invoke_function=None):
        """Setup config, execute function"""
        ctx = self._create_context()
        with self._setup():
            self._execute(ctx, invoke_function)

    def _execute(self, ctx: Context, invoke_function: T.Optional[str]):
        for handler in ctx.cfg.handlers:
            condition = handler.get_condition(ctx.gh.event_name)
            if not condition.check(ctx):
                L.verbose1("%s: skip handler for %s because of condition failed", handler.name, ctx.gh.event_name)
                continue
            if invoke_function:
                func_name = invoke_function
            else:
                func_name = f"handle_{ctx.gh.event_name}"
            func = handler.get_function(func_name)
            if not func:
                L.verbose1("%s: function %s not found", handler.name, func_name)
                continue
            L.verbose1("%s: execute %s", handler.name, func_name)
            try:
                with ctx.gh.check(f"dandori::{func_name}"):
                    r = func(ctx)
            except exception.Cancel:
                ctx.gh.cancel()
            except Exception as e:
                print(f"::error::{e}")
                raise
            if isinstance(r, dict):
                ctx.resp.append_dict(handler.name, r)
            elif isinstance(r, dandori.response.Response):
                ctx.resp.append(handler.name, r)
            else:
                ctx.resp.append_dict(handler.name, {})

    def _create_context(self) -> Context:
        if self._local_mode:
            gh = GitHubMock()
        else:
            gh = GitHub()  # type: ignore
        config = ConfigLoader().load(self._cfg_path)
        if self._local_mode:
            config.local = True
        config.options.merge_update(self._options)
        L.verbose3("Options: %s", config.options)
        ops = Operation()
        resp = dandori.response.Responses()
        return Context(gh=gh, cfg=config, ops=ops, resp=resp)

    @contextlib.contextmanager
    def _setup(self):
        tempdir = env.tempdir()
        sys.meta_path.append(HandlerFinder)
        try:
            yield
        finally:
            tempdir.cleanup()
        sys.meta_path.remove(HandlerFinder)



from __future__ import annotations

import dataclasses
import typing as T

import dandori.config
import dandori.gh
import dandori.ops
import dandori.response


@dataclasses.dataclass
class Context:
    gh: T.Union[dandori.gh.GitHub, dandori.gh.GitHubMock]
    cfg: dandori.config.Config
    ops: dandori.ops.Operation
    resp: dandori.response.Responses



from __future__ import annotations

import os
import tempfile

TEMP_DIR = None


def tempdir() -> tempfile.TemporaryDirectory:
    """create temporary directory and cache. need to cleanup somewhere"""
    global TEMP_DIR  # pylint: disable=global-statement
    if TEMP_DIR is None or not os.path.exists(TEMP_DIR.name):
        basedir = os.environ.get("DANDORI_TEMP_DIR")
        TEMP_DIR = tempfile.TemporaryDirectory(prefix="dandori_", dir=basedir)
    return TEMP_DIR



from __future__ import annotations

import logging
import sys
import time
import typing as T

SILENT = logging.CRITICAL + 1
CRITICAL = logging.CRITICAL
ERROR = logging.ERROR
WARNING = logging.WARNING
INFO = logging.INFO
VERBOSE1 = logging.INFO - 1
VERBOSE2 = logging.INFO - 2
VERBOSE3 = logging.INFO - 3
DEBUG = logging.DEBUG
NOTSET = logging.NOTSET

_LEVELS = {
    SILENT: ("SILENT", "S"),
    CRITICAL: ("CRITICAL", "C"),
    ERROR: ("ERROR", "E"),
    WARNING: ("WARNING", "W"),
    INFO: ("INFO", "I"),
    VERBOSE1: ("VERBOSE1", "V"),
    VERBOSE2: ("VERBOSE2", "V2"),
    VERBOSE3: ("VERBOSE3", "V3"),
    DEBUG: ("DEBUG", "D"),
    NOTSET: ("NOTSET", "N"),
}

DEFAULT_FORMAT = "%(prefix)s%(message)s"


class ModuleLogFormatter(logging.Formatter):
    """Custom Logger"""

    converter = time.gmtime  # asctime as a UTC time

    def __init__(self, *args, **kwargs):
        """logging formatter"""
        super().__init__(*args, **kwargs)
        self.fmt_str = kwargs.get("fmt")

    def format(self, record):
        """custom format"""
        levelname = _LEVELS.get(record.levelno)
        if levelname:
            record.prefix = levelname[0].capitalize() + ": " if record.levelno > INFO else ""
            record.levelname = levelname[0]
            record.shortlevelname = levelname[1]
        else:
            record.shortlevelname = ""

        s = super().format(record)
        return s


class Logger:
    """Simple Logger wrapper"""

    def __init__(self, name):
        """Simple Logger wrapper"""
        self._logger = logging.getLogger(name)

    def __getattr__(self, name):
        """Redirect to logging.Logger"""
        return getattr(self._logger, name)

    def verbose1(self, msg, *args, **kwargs):
        """additional log level"""
        if self._logger.isEnabledFor(VERBOSE1):
            self._logger._log(VERBOSE1, msg, args, **kwargs)  # pylint: disable=protected-access

    def verbose2(self, msg, *args, **kwargs):
        """additional log level"""
        if self._logger.isEnabledFor(VERBOSE2):
            self._logger._log(VERBOSE2, msg, args, **kwargs)  # pylint: disable=protected-access

    def verbose3(self, msg, *args, **kwargs):
        """additional log level"""
        if self._logger.isEnabledFor(VERBOSE3):
            self._logger._log(VERBOSE3, msg, args, **kwargs)  # pylint: disable=protected-access


def get_logger(name: T.Optional[str] = None):
    """Return logger"""
    _init_root_logger()
    if name is None:
        name = "dandori"
    return Logger(name)


def get_level() -> int:
    """get current log level"""
    return get_logger().level


def set_level(level: T.Union[str, int]):
    """set log level"""
    lvl = level
    if isinstance(lvl, str):
        lvl = _level_name_to_num(lvl)
    if not isinstance(lvl, int):
        raise ValueError(f"Unknown level: {level}")
    get_logger().setLevel(lvl)
    if lvl <= DEBUG:
        set_format("[%(pathname)s(%(lineno)s):%(funcName)s] " + DEFAULT_FORMAT)


def set_format(fmt):
    """Change logging format"""
    logger = get_logger()
    logger.handlers[0].setFormatter(ModuleLogFormatter(fmt=fmt))


def _level_name_to_num(levelname: str):
    """Find integer level by its name"""
    n = levelname.upper()
    for levelno, (name, short_name) in _LEVELS.items():
        if n in (name, short_name):
            return levelno
    return None


def _init_root_logger():
    if hasattr(_init_root_logger, "__initialized"):
        return
    # initialize only once
    logger = logging.getLogger("dandori")
    handler = logging.StreamHandler(sys.stdout)
    handler.setFormatter(ModuleLogFormatter(fmt=DEFAULT_FORMAT))
    handler.setLevel(NOTSET)
    logger.addHandler(handler)
    logger.propagate = False
    logger.setLevel(INFO)
    setattr(_init_root_logger, "__initialized", True)



from __future__ import annotations

import dataclasses
import importlib
import pathlib
import pprint
import shutil
import typing as T

from box import Box

import dandori.log
from dandori import env, exception

if T.TYPE_CHECKING:
    from dandori.context import Context


L = dandori.log.get_logger(__name__)


class Condition:
    def __init__(self, src: T.Optional[dict] = None):
        """Represents handler condition to execute function or not

        Args:
            src (T.Optional[dict], optional): configuration object. Defaults to None.
        """
        self._src = src

    def check(self, ctx: Context):  # pylint: disable=unused-argument
        """Check if execute handler or not"""
        L.warning("Condition check does not implemented yet")
        return True


class HandlerLoader:
    def __init__(self, name: str, path: pathlib.Path):
        """Handler loader for local path"""
        self._module_name = name
        self._path = path

    @property
    def module_name(self):
        """module name. dynamically loaded full package name is dandori.handlers.{module_name}"""
        return self._module_name

    def load_module(self):
        """load module"""
        return importlib.import_module(f"dandori.handlers.{self._module_name}")

    def deploy_package(self):
        """Retrieve package files and place it to temporal package directory"""
        rootdir = pathlib.Path(env.tempdir().name).joinpath("handlers")
        path = pathlib.Path(self._path)
        if not path.exists():
            raise ValueError(f"{path} does not exist")
        if path.is_file():
            shutil.copy(path, rootdir.joinpath(self._module_name + ".py"))
        else:
            pkgdir = rootdir.joinpath(self._module_name)
            if pkgdir.exists():
                shutil.rmtree(pkgdir)
            shutil.copytree(path, pkgdir)


class Handler:
    """Load user module/package/script and run specific function"""

    def __init__(self, loader: HandlerLoader):
        """user defined script package/module"""
        self._loader = loader
        self._mod = None
        self._loader.deploy_package()

    @property
    def name(self):
        """module/package name in configuration or automatically named"""
        return self._loader.module_name

    def get_function(self, func_name: str):
        """Run function corresponding to the action name

        Args:
            func_name (str): function name to get
        """
        self._load_module()
        return getattr(self._mod, func_name, None)

    def get_condition(self, event_name: str):
        """Get condition object correspond to event"""
        self._load_module()
        if not hasattr(self._mod, "conditions"):
            return Condition()
        else:
            conditions = self._mod.conditions()  # type: ignore
            return Condition(conditions[event_name])

    def _load_module(self):
        if self._mod is None:
            self._mod = self._loader.load_module()


@dataclasses.dataclass
class Config:
    handlers: list[Handler]
    local: bool = False  # Run in local mode or not
    cwd: pathlib.Path = pathlib.Path(".").absolute()  # current directory at instance generation point
    options: Box = dataclasses.field(default_factory=Box)


class ConfigLoader:
    """Load local/remote configurations and script/package"""

    def load(self, config_file):
        """load configuration file"""
        if config_file:
            path = pathlib.Path(config_file)  # ensure Path
            if not path.exists():
                raise exception.DandoriError(f"File not found: {path}")
        else:
            path = pathlib.Path("dandori.yaml")
            if not path.exists():
                path = pathlib.Path("pyproject.toml")
                if not path.exists():
                    raise exception.DandoriError(
                        "config file not found. You need to create dandori.yaml or write configs in pyproject.toml"
                    )

        if path.suffix in (".yaml", ".yml"):
            conf = Box.from_yaml(filename=str(path))
        elif path.suffix == ".toml":
            conf = Box.from_toml(filename=str(path))
            if "dandori" in conf.get("tool", {}):  # pyproject.toml support
                conf = conf["tool"]
            if "dandori" not in conf:
                raise ValueError(f"{path}: dandori section not found in your config file")
            conf = conf["dandori"]
        else:
            raise exception.DandoriError(f"Unsupported configuration format: {path}")
        L.debug("Config: %s", pprint.pformat(conf))
        return Config(handlers=self._parse_handlers(conf, path.parent), options=self._parse_options(conf))

    def _parse_handlers(self, conf: dict, basedir: pathlib.Path):
        rootdir = pathlib.Path(env.tempdir().name).joinpath("handlers")
        rootdir.mkdir(exist_ok=True)
        rootdir.joinpath("__init__.py").touch()
        handlers = []
        for i, d in enumerate(conf.get("handlers", [])):
            if isinstance(d, str):
                name = f"package_{i}"
                path = pathlib.Path(d)
            elif isinstance(d, dict):
                name = d.get("name", f"package_{i}")
                path = pathlib.Path(d["path"])
            else:
                raise ValueError(f"handlers.{i} must be dict or str")
            if not path.is_absolute():
                path = basedir.joinpath(path)
            loader = HandlerLoader(name=name, path=path)
            handlers.append(Handler(loader))
            L.verbose3("Add handlers: %s", name)
        return handlers

    def _parse_options(self, conf: Box):
        return conf.get("options", Box())



from __future__ import annotations

import contextlib
import os
import pathlib
import time
import typing as T
import urllib.error

from box import Box
from ghapi.all import GhApi

import dandori.exception
import dandori.log
import dandori.ops

HTTPError = urllib.error.HTTPError
URLError = urllib.error.URLError

L = dandori.log.get_logger(__name__)


class GitHub:
    def __init__(self):
        """Read GitHub Actions info automatically, and provide some convenient methods"""
        self._path = pathlib.Path(os.environ["GITHUB_EVENT_PATH"])
        self.repository: str = os.environ["GITHUB_REPOSITORY"]
        self.event_name: str = os.environ["GITHUB_EVENT_NAME"]
        self.sha: str = os.environ["GITHUB_SHA"]
        self.ref: str = os.environ["GITHUB_REF"]
        self.workflow: str = os.environ["GITHUB_WORKFLOW"]
        self.action: str = os.environ["GITHUB_ACTION"]
        self.actor: str = os.environ["GITHUB_ACTOR"]
        self.job: str = os.environ["GITHUB_JOB"]
        self.run_number: int = int(os.environ["GITHUB_RUN_NUMBER"])
        self.run_id: int = int(os.environ["GITHUB_RUN_ID"])
        self.payload: Box = Box()
        if self._path.exists():
            self.payload = Box.from_json(filename=str(self._path))
        self.api = GhApi(owner=self.owner, repo=self.name)
        self._pull_request = None
        #
        if self.event_name == "issue_comment":
            if self.is_pull_request():
                self.event_name = "pull_request_comment"
                self._checkout_pull_request_branch()

    @property
    def owner(self) -> str:
        """Return owner: repository=owner/name"""
        return self.repository.split("/")[0]

    @property
    def name(self) -> str:
        """Return repository name: repository=owner/name"""
        return self.repository.split("/")[1]

    @property
    def issue_number(self) -> T.Optional[int]:
        """issue number if exists"""
        if "issue" in self.payload:
            return self.payload["issue"]["number"]
        elif "pull_request" in self.payload:
            return self.payload["pull_request"]["number"]
        return None

    def pull_request(self, number: int = None) -> Box:
        """Get pull request details"""
        try:
            if number is None:
                if self.is_pull_request():
                    if self._pull_request is None:
                        self._pull_request = Box(self.api.pulls.get(self.issue_number))
                    return self._pull_request
                else:
                    return self._pull_request
            else:
                return Box(self.api.pulls.get(number))
        except HTTPError as e:
            if e.code == 404:
                return Box()
            raise

    def is_pull_request(self):
        """event is related to pull request (pull request, pull request comment)"""
        if self.event_name == "pull_request":
            return True
        elif "pull_request" in self.payload.get("issue", {}):
            return True
        return False

    def latest_release_tag(self) -> str:
        """Return latest relesed tag"""
        try:
            resp = self.api.get_latest_release()
        except HTTPError as e:
            if e.code == 404:
                return ""
        return resp.tag_name

    def has_tag(self, tag: str) -> bool:
        """Check a tag already exists"""
        results = self.api.list_tags(tag)
        refs = [x.ref for x in results]
        return f"refs/tags/{tag}" in refs

    def has_label(self, name: T.Union[str, frozenset[str], set[str], list[str], tuple[str]]) -> bool:
        """Return True if issue/pull_request has an label"""
        labels = self.payload.get("issue", {}).get("labels")
        if not labels:
            if self.is_pull_request():
                pr = self.pull_request()
                if pr and "labels" in pr:
                    labels = pr["labels"]
        if labels:
            for label in labels:
                if isinstance(name, str) and label["name"] == name:
                    return True
                elif isinstance(name, (list, tuple, set, frozenset)) and label["name"] in name:
                    return True
        return False

    def add_or_create_label(self, name, color=None):
        """Add a label to issue"""
        # call api manualy. See related issue: https://github.com/fastai/ghapi/issues/69
        # check the label exists and create with color
        try:
            label = self.api.issues.get_label(name)
        except HTTPError as e:
            if e.code == 404:
                # create label
                label = self.api.issues.create_label(name, color)
        self.api.__call__(
            path=self.api.issues.add_labels.path,
            verb=self.api.issues.add_labels.verb,
            route=dict(issue_number=self.issue_number),
            data=dict(labels=[label.name]),
        )

    def create_comment(self, body: str):
        """Create comment to its issue/pull_request"""
        if not self.issue_number:
            raise ValueError("issue number not found.")
        self.api.issues.create_comment(self.issue_number, body)

    def comment_body(self) -> str:
        """Get comment body if exists"""
        return self.payload.get("comment", {}).get("body", "")

    def create_release(self, *args, **kwargs):
        """Shorthand for api.create_release"""
        self.api.create_release(*args, **kwargs)

    def cancel(self):
        """Cancel this workflow"""
        self.api.actions.cancel_workflow_run(self.run_id)
        time.sleep(10)

    def list_checks(self, sha=None, name=None, status=None):
        """Get check runs and return"""
        if self.is_pull_request():
            pr = self.pull_request()
            sha = pr.head.sha
        else:
            sha = self.sha
        kwargs = {"ref": sha}
        if name is not None:
            kwargs["check_name"] = name
        if status is not None:
            kwargs["status"] = status
        checks = self.api.checks.list_for_ref(per_page=100, **kwargs)
        return [Box(x) for x in checks.check_runs]

    @contextlib.contextmanager
    def check(self, name: str, sha=None):
        """Some proc with GitHub Checks API"""
        if sha is None:
            if self.is_pull_request():
                sha = self.pull_request()["head"]["sha"]
            elif self.sha:
                sha = self.sha
        try:
            conclusion = "success"
            check = self.api.checks.create(name=name, head_sha=sha, status="in_progress")
            yield
        except dandori.exception.Cancel:
            conclusion = "cancelled"
            raise
        except Exception:
            conclusion = "failure"
            raise
        finally:
            self.api.checks.update(name=name, check_run_id=check.id, status="completed", conclusion=conclusion)

    def _checkout_pull_request_branch(self):
        if pathlib.Path(".git").is_dir():
            ops = dandori.ops.Operation()
            pr = self.pull_request()
            ref = pr["merge_commit_sha"]
            L.info("Checkout to merge commit of PR #%d: %s", self.issue_number, ref)
            ops.run(["git", "fetch", "origin", f"+{ref}:refs/remotes/origin/merge_commit"])
            ops.run(["git", "checkout", "--force", "-B", "merge_commit", "refs/remotes/origin/merge_commit"])


class GitHubMock:
    def __init__(self, chain=None):
        """GitHub API Mock"""
        self._chain = [] if chain is None else chain

    def __call__(self, *args, **kwargs):
        """call outputs what api called"""
        call_chain = ".".join(self._chain)
        L.info("[Call GitHub API] %s(%s, %s)", call_chain, args, kwargs)

    @contextlib.contextmanager
    def check(self, *args, **kwargs):
        """log check"""
        L.info("[Call GitHub API] check(%s, %s)", args, kwargs)
        yield

    def __getattr__(self, name):
        return GitHubMock(self._chain + [name])



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['dandori']

package_data = \
{'': ['*']}

install_requires = \
['ghapi>=0.1.17,<0.2.0',
 'python-box>=5.3.0,<6.0.0',
 'ruamel.yaml>=0.17.10,<0.18.0',
 'toml>=0.10.2,<0.11.0']

entry_points = \
{'console_scripts': ['dandori = dandori.__main__:main']}

setup_kwargs = {
    'name': 'dandori',
    'version': '0.1.14',
    'description': 'GitHub Actions with Python',
    'long_description': '# dandori: GitHub Actions with Python\n\ndandori runs on your Actions, and automate workflow with using Python.\n\n**Current Status is Super Early Alpha. DO NOT USE IT in your production repository.**\n\n\n## How to Use\n\nFirst, You need to define workflow.\nYou can hook any [events](https://docs.github.com/en/actions/reference/events-that-trigger-workflows) without manual/scheduled workflow such as `pull_request`, `push` or `pull_request_review`.\n\n```yaml\nname: dandori_action\non: [pull_request, issue_comment]\n\njobs:\n  run:\n    runs-on: ubuntu-latest\n    steps:\n      - uses: actions/checkout@v2\n      - uses: actions/setup-python@v2\n        with:\n          python-version: \'3.9\'\n      - run: pip install dandori\n      - run: dandori run\n        env:\n          GITHUB_TOKEN: ${{ secrets.GITHUB_TOKEN }}\n```\n\nNext, you can write your Python script or package on your repo, like `dandori_handler.py` or \'dandori_handlers/\'. dandori automatically import your code and run handler functions defined in your code:\n\n```py\n## dandori_handler.py\n\ndef handle_pull_request(ctx):\n    if ctx.gh.payload.action == \'synchronize\':\n        ctx.gh.create_comment("You pushed new commits!!")\n\n\ndef handle_pull_request_comment(ctx):\n    """It\'s a special handler type, issue_comment event on PR"""\n    comment_body = ctx.gh.comment_body().strip()\n    if comment_body.startswith(\'/some-command\'):\n        some_code_as_you_like()\n```\n\nIf you want more than one file, you need to make a package:\n\n```py\n## handlers/__init__.py\n# Must be relative imports\nfrom .pull_request import handle_pull_request\nfrom .issue import handle_issue\n\n## handlers/pull_request.py\ndef handle_pull_request(ctx):\n    ...\n\n## -- handlers/issue.py\ndef handle_issue(ctx):\n    ...\n```\n\n## Configuration\n\ndandori supports `pyproject.toml`, or make any toml file as you like (default is `dandori.toml`).\n\nIn pyproject.toml, write config in `tool.dandori` section:\n\n```toml\n# pyproject.toml\n[tool.dandori]\nhandlers = [\'path/to/handler\']\n```\n\nIn independent toml file, write config in `dandori` section:\n\n```toml\n# dandori.toml\n[dandori]\nhandlers = [\'path/to/handler\']\n```\n\n\n## Use case\n\n### Share CI code with multiple repo:\n\n\n```yaml\nname: dandori_action\non: [pull_request, issue_comment]\n\njobs:\n  run:\n    runs-on: ubuntu-latest\n    steps:\n      - uses: actions/checkout@v2\n      - uses: actions/checkout@v2\n        with:\n          repository: your/dandori-handler\n          ref: v1  # something you need\n          ssh-key: ${{ secrets.your_repo_key }}\n          path: dandori-handler\n      - uses: actions/setup-python@v2\n        with:\n          python-version: \'3.9\'\n      - run: pip install dandori\n      - run: dandori run -f dandori-handler/dandori.toml\n        env:\n          GITHUB_TOKEN: ${{ secrets.GITHUB_TOKEN }}\n```\n\n### Use third party package in handler\n\nYour handler module or package will be imported dynamically, so you can install any library in the "(virtual)env" same as dandori installed.\n\nMost simple cale, install dandori with other library:\n\n```\n# Install libraries with pip\n- run: pip install dandori requests python-dateutil\n```\n\nIf you want use just a "command" and not to use global env, use `ctx.ops.run_venv()`:\n\n```py\ndef handle_pull_request(ctx):\n    ctx.ops.run_venv(["pip", "install", "twine"])\n    ctx.ops.run_venv([\'twine\', \'upload\', \'dist/*\'])\n```\n\nOr dynamically install it and use it:\n\n```py\nimport importlib\n\ndef handle_pull_request(ctx):\n    ctx.ops.run(["pip", "install", "requests"])\n    requests = importlib.import_module(\'requests\')\n```',
    'author': 'Hiroyuki Tanaka',
    'author_email': 'aflc0x@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/roy-ht/dandori',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
